"use strict";
// Copyright IBM Corp. and LoopBack contributors 2020. All Rights Reserved.
// Node module: @loopback/example-webpack
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
Object.defineProperty(exports, "__esModule", { value: true });
exports.GreetingApplication = exports.main = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
/**
 * A strongly-typed binding key for current date
 */
const CURRENT_DATE = core_1.BindingKey.create('currentDate');
/**
 * A strongly-typed binding key for current user
 */
const CURRENT_USER = core_1.BindingKey.create('currentUser');
/**
 * A strongly-typed binding key for current language
 */
const CURRENT_LANGUAGE = core_1.BindingKey.create('currentLanguage');
/**
 * A strongly-typed binding key for `GreetingService`
 */
const GREETING_SERVICE = core_1.BindingKey.create('services.GreetingService');
/**
 * A factory function to return the current date
 */
const getCurrentDate = () => new Date();
class ChineseGreeter {
    constructor() {
        this.language = 'zh';
    }
    greet(name) {
        return `你好，${name}！`;
    }
}
class EnglishGreeter {
    constructor() {
        this.language = 'en';
    }
    greet(name) {
        return `Hello, ${name}!`;
    }
}
/**
 * A class with dependency injection
 */
let GreetingService = class GreetingService {
    // Constructor based injection
    constructor(greetersView) {
        this.greetersView = greetersView;
    }
    async greet(language, userName) {
        // Get current date
        const date = await this.now();
        // Get current list of greeters
        const greeters = await this.greetersView.values();
        for (const greeter of greeters) {
            if (greeter.language === language) {
                const msg = greeter.greet(userName);
                return `[${date.toISOString()}] (${language}) ${msg}`;
            }
        }
        return `[${date.toISOString()}] Hello, ${userName}!`;
    }
};
tslib_1.__decorate([
    core_1.inject.getter(CURRENT_DATE),
    tslib_1.__metadata("design:type", Function)
], GreetingService.prototype, "now", void 0);
tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.inject)(CURRENT_LANGUAGE)),
    tslib_1.__param(1, (0, core_1.inject)(CURRENT_USER)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String]),
    tslib_1.__metadata("design:returntype", Promise)
], GreetingService.prototype, "greet", null);
GreetingService = tslib_1.__decorate([
    tslib_1.__param(0, core_1.inject.view((0, core_1.filterByTag)('greeter'))),
    tslib_1.__metadata("design:paramtypes", [core_1.ContextView])
], GreetingService);
async function main() {
    const app = new GreetingApplication();
    // Add EnglishGreeter for now
    app.addGreeter('greeters.EnglishGreeter', EnglishGreeter);
    // Set the current user to `John` (a constant value)
    await app.greetInEnglish();
    // Now add ChineseGreeter
    app.addGreeter('greeters.ChineseGreeter', ChineseGreeter);
    await app.greetInChineseThenEnglish();
    return app.greetings;
}
exports.main = main;
class GreetingApplication extends core_1.Application {
    constructor() {
        super();
        this.greetings = [];
        // Bind greeting service
        this.bind(GREETING_SERVICE).toClass(GreetingService);
        // Set the current date to a factory function
        this.bind(CURRENT_DATE).toDynamicValue(getCurrentDate);
    }
    addGreeter(key, cls) {
        this.bind(key).toClass(cls).tag('greeter');
    }
    async greetInEnglish() {
        const { greetingService, requestCtx } = await this.createRequest('John', 'zh');
        // Invoke `greet` as a method
        // '(en) Hello, Jane'
        let greeting = await greetingService.greet('en', 'Jane');
        this.greetings.push(greeting);
        // Use `invokeMethod` to apply method injection
        // 'Hello, John!' - no matching greeter is found
        greeting = await (0, core_1.invokeMethod)(greetingService, 'greet', requestCtx);
        this.greetings.push(greeting);
        requestCtx.close();
    }
    async greetInChineseThenEnglish() {
        const { greetingService, requestCtx } = await this.createRequest('John', 'zh');
        let greeting = await (0, core_1.invokeMethod)(greetingService, 'greet', requestCtx);
        // '(zh) 你好，John！',
        this.greetings.push(greeting);
        // Switch the current language to `en`
        requestCtx.bind(CURRENT_LANGUAGE).to('en');
        // Switch the current user to `Jane`
        requestCtx.bind(CURRENT_USER).to('Jane');
        greeting = await (0, core_1.invokeMethod)(greetingService, 'greet', requestCtx);
        // '(en) Hello, Jane!'
        this.greetings.push(greeting);
        requestCtx.close();
    }
    async createRequest(user, language) {
        const requestCtx = new core_1.Context(this, `request-${GreetingApplication.requestCounter++}`);
        requestCtx.bind(CURRENT_USER).to(user);
        requestCtx.bind(CURRENT_LANGUAGE).to(language);
        // Get an instance of the greeting service
        const greetingService = await requestCtx.get(GREETING_SERVICE);
        return { greetingService, requestCtx };
    }
}
exports.GreetingApplication = GreetingApplication;
GreetingApplication.requestCounter = 0;
/* istanbul ignore if */
if (require.main === module) {
    main()
        .then(greetings => greetings.forEach(g => console.log(g)))
        .catch(err => {
        console.error(err);
        process.exit(1);
    });
}
//# sourceMappingURL=dependency-injection.js.map